with Ada.Text_IO; -- for our usual "printf" (Put[_Line])
with Museum;

package body Tourists is

   --  we use a modular type that wraps around
   --  in order for task instances to keep count of
   --  their infinite number of visits to the Museum
   Limit : constant Positive := 10;
   type Counter_T is mod Limit;

   --+---------------------------
   task body Visitor_T is
      use Museum,
          Ada.Text_IO;

      --  each process instance keeps a running
      --  counter of the number of visits paid to
      --  the very beautiful museum guarded by
      --  Museum.Guardian
      --+
      --  Visit never overflows but just wraps around at Limit
      Visit : Counter_T := 0;

   begin
      loop
         Visit := Visit + 1;
         Put_Line
           ("Visitor #" & Positive'Image (Id)
            & " about to queue for museum visit #"
            & Counter_T'Image (Visit));

         --  the process applies for entrance
         Guardian.Enter;
         --  permission granted

         Put_Line
           ("          Visitor #" & Positive'Image (Id)
            & " is granted permission to enter.");
         --  at this point, the process sleeps,
         --  dines and then repeats the whole cycle

         --  a relative-duration visit
         delay Duration (Interval);

         --  the process notifies exit
         Guardian.Leave;

         --  the process mind its own business before returning
         delay Duration (Interval * 2);
      end loop;
   end Visitor_T;

end Tourists;
